@@ -3,10 +3,8 @@ use strict;
 use warnings;
 package App::Spec::Run;
 use 5.010;
-our $VERSION = '0.003_002'; # TRIAL VERSION
+our $VERSION = '0.004_001'; # TRIAL VERSION
 
-use List::Util qw/ any /;
-use Data::Dumper;
 use App::Spec::Run::Validator;
 use App::Spec::Run::Response;
 use Getopt::Long qw/ :config pass_through bundling /;
@@ -17,24 +15,50 @@ has spec => ( is => 'ro' );
 has options => ( is => 'rw' );
 has parameters => ( is => 'rw', default => sub { +{} } );
 has commands => ( is => 'rw' );
-has runmode => ( is => 'rw', default => 'normal' );
-has errors => ( is => 'rw' );
+has argv => ( is => 'rw' );
+has argv_orig => ( is => 'rw' );
+#has runmode => ( is => 'rw', default => 'normal' );
+has validation_errors => ( is => 'rw' );
 has op => ( is => 'rw' );
 has cmd => ( is => 'rw' );
-has response => ( is => 'rw' );
-has validation_errors => ( is => 'rw' );
+has response => ( is => 'rw', default => sub { App::Spec::Run::Response->new } );
+has subscribers => ( is => 'rw', default => sub { +{} } );
+
+my %EVENTS = (
+    print_output => 1,
+    global_options => 1,
+);
 
 sub process {
     my ($self) = @_;
 
-    my $res = $self->response;
-    unless ($res) {
-        $res = App::Spec::Run::Response->new;
-        $self->response($res);
+    my $plugins = $self->spec->plugins || [];
+    for my $plugin (@$plugins) {
+        $plugin->init_run($self);
+    }
+    my @callbacks;
+    my $subscriber_events = $self->subscribers;
+    for my $key (qw/ global_options print_output /) {
+        my $subscribers = $subscriber_events->{ $key };
+        for my $sub (@$subscribers) {
+            my $plugin = $sub->{plugin};
+            my $method = $sub->{method};
+            my $callback = sub {
+                $plugin->$method( run => $self, @_);
+            };
+            push @callbacks, $callback;
+        }
+        $self->response->add_callbacks($key => \@callbacks);
+    }
+
+    my $argv = $self->argv;
+    unless ($argv) {
+        $argv = \@ARGV;
+        $self->argv($argv);
+        $self->argv_orig([ @$argv ]);
     }
 
     my $completion_parameter = $ENV{PERL5_APPSPECRUN_COMPLETION_PARAMETER};
-    $self->check_help;
 
     my %option_specs;
     my %param_specs;
@@ -53,11 +77,10 @@ sub process {
             param_specs => \%param_specs,
         });
         my %errs;
-        my ($ok) = $opt->process( \%errs, type => "parameters", app => $self );
-        $ok &&= $opt->process( \%errs, type => "options", app => $self );
-        $self->errors(\%errs) if not $ok;
+        my $ok = $opt->process( $self, \%errs );
         unless ($ok) {
             $self->validation_errors(\%errs);
+            # if we are in completion mode, some errors might be ok
             if (not $completion_parameter) {
                 $self->error_output;
             }
@@ -86,6 +109,7 @@ sub run {
 
     $self->process;
 
+#    $self->event_processed;
     $self->finish;
 
 }
@@ -97,16 +121,14 @@ sub run_op {
 
 sub out {
     my ($self, $text) = @_;
-    my $res = $self->response;
-    $text .= "\n" unless $text =~ m/\n\z/;
-    $res->add_output($text);
+    $text .= "\n" if (not ref $text and $text !~ m/\n\z/);
+    $self->response->add_output($text);
 }
 
 sub err {
     my ($self, $text) = @_;
-    my $res = $self->response;
-    $text .= "\n" unless $text =~ m/\n\z/;
-    $res->add_error($text);
+    $text .= "\n" if (not ref $text and $text !~ m/\n\z/);
+    $self->response->add_error($text);
 }
 
 sub halt {
@@ -174,7 +196,7 @@ sub completion_output {
 
 sub error_output {
     my ($self) = @_;
-    my $errs = $self->errors;
+    my $errs = $self->validation_errors;
     my @error_output;
     for my $key (sort keys %$errs) {
         my $errors = $errs->{ $key };
@@ -186,6 +208,7 @@ sub error_output {
             }
         }
         else {
+            require Data::Dumper;
             my $err = Data::Dumper->Dump([$errs], ['errs']);
             push @error_output, $err;
         }
@@ -197,8 +220,7 @@ sub error_output {
     );
     $self->err($help);
     for my $msg (@error_output) {
-        $self->colorize
-            and $msg = $self->colored('err', [qw/ error /], $msg);
+        $msg = $self->colored('err', [qw/ error /], $msg);
         $self->err("$msg\n");
     }
     $self->halt(1);
@@ -244,11 +266,11 @@ sub process_parameters {
         my $required = $p->required;
         my $value;
         if ($multiple) {
-            $value = [@ARGV];
-            @ARGV = ();
+            $value = [@{ $self->argv }];
+            @{ $self->argv } = ();
         }
         else {
-            $value = shift @ARGV;
+            $value = shift @{ $self->argv };
         }
         $parameters->{ $name } = $value;
         $param_specs->{ $name } = $p;
@@ -258,6 +280,7 @@ sub process_parameters {
 sub process_input {
     my ($self, %args) = @_;
     my %options;
+    $self->options(\%options);
     my @cmds;
     my $spec = $self->spec;
     my $option_specs = $args{option_specs};
@@ -266,6 +289,8 @@ sub process_input {
     my $global_parameters = $spec->parameters;
     my @getopt = $spec->make_getopt($global_options, \%options, $option_specs);
     GetOptions(@getopt);
+    $self->event_globaloptions;
+    my $op = $self->op;
 
     $self->process_parameters(
         parameter_list => $global_parameters,
@@ -275,12 +300,11 @@ sub process_input {
 
 
     my $commands = $spec->subcommands;
-    my $op;
+    my $opclass = $self->spec->class;
     my $cmd_spec;
     my $subcommand_required = 1;
     while (keys %$commands) {
-        my @k = keys %$commands;
-        my $cmd = shift @ARGV;
+        my $cmd = shift @{ $self->argv };
         if (not defined $cmd) {
             if (not $op or $subcommand_required) {
                 $self->err($spec->usage(
@@ -290,7 +314,7 @@ sub process_input {
                         subcommands => 1,
                     },
                 ));
-                $self->err( $self->error("Missing subcommand(s)") );
+                $self->err( $self->colorize_error("Missing subcommand(s)") );
                 $self->halt(1);
             }
             last;
@@ -303,7 +327,7 @@ sub process_input {
                     subcommands => 1,
                 },
             ));
-            $self->err( $self->error("Unknown subcommand '$cmd'") );
+            $self->err( $self->colorize_error("Unknown subcommand '$cmd'") );
             $self->halt(1);
             last;
         };
@@ -313,7 +337,8 @@ sub process_input {
         GetOptions(@getopt);
         push @cmds, $cmd;
         $commands = $cmd_spec->subcommands || {};
-        $op = $cmd_spec->op if $cmd_spec->op;
+        $op = '::' . $cmd_spec->op if $cmd_spec->op;
+        $opclass = $cmd_spec->class if $cmd_spec->class;
 
         $self->process_parameters(
             parameter_list => $cmd_spec->parameters,
@@ -333,11 +358,14 @@ sub process_input {
                 $self->halt(1);
             }
             else {
-                $op = "execute";
+                $op = "::execute";
             }
         }
         $self->commands(\@cmds);
         $self->options(\%options);
+        if ($op =~ m/^::/) {
+            $op = $opclass . $op;
+        }
         $self->op($op);
         return $op;
     }
@@ -345,7 +373,7 @@ sub process_input {
     return;
 }
 
-sub error {
+sub colorize_error {
     my ($self, $msg) = @_;
     $msg = $self->colored('err', [qw/ error /], $msg) . "\n";
 }
@@ -359,87 +387,278 @@ sub colored {
     return $msg;
 }
 
-sub check_help {
+sub subscribe {
+    my ($self, %args) = @_;
+
+    for my $event (sort keys %args) {
+        next unless exists $EVENTS{ $event };
+        my $info = $args{ $event };
+        push @{ $self->subscribers->{ $event } }, $info;
+    }
+
+}
+
+sub event_globaloptions {
     my ($self) = @_;
-    GetOptions(
-        "help|h" => \my $help,
-    );
 
-    my $op;
-    if ($self->spec->has_subcommands) {
-        if ($help and (not @ARGV or $ARGV[0] ne "help")) {
-            # call subcommand 'help'
-            unshift @ARGV, "help";
-        }
+    my $subscribers = $self->subscribers->{global_options};
+    for my $sub (@$subscribers) {
+        my $plugin = $sub->{plugin};
+        my $method = $sub->{method};
+        $plugin->$method( run => $self);
     }
-    else {
-        if ($help) {
-            $op = "cmd_help";
-            unshift @ARGV, "--help";
-        }
-    }
-
-    $self->op($op);
 }
 
-
-sub cmd_help {
-    my ($run) = @_;
-    my $spec = $run->spec;
-    my $cmds = $run->commands;
-    shift @$cmds;
-    my $help = $spec->usage(
-        commands => $cmds,
-        colored => $run->colorize_code,
-    );
-    $run->out($help);
-}
-
-sub cmd_self_completion {
-    my ($run) = @_;
-    my $options = $run->options;
-    my $shell = $options->{zsh} ? "zsh" : $options->{bash} ? "bash" : '';
-    unless ($shell) {
-        my $ppid = getppid();
-        chomp($shell = `ps --no-headers -o cmd $ppid`);
-        $shell =~ s/.*\W(\w*sh).*$/$1/; #handling case of '-zsh' or '/bin/bash'
-                                        #or bash -i -rs
-    }
-    unless (any { $_ eq $shell } qw/ bash zsh / ) {
-        die "Specify which shell, '$shell' not supported";
-    }
-    my $spec = $run->spec;
-    my $completion = $spec->generate_completion(
-        shell => $shell,
-    );
-    $run->out($completion);
-}
-
-sub cmd_self_pod {
-    my ($run) = @_;
-    my $spec = $run->spec;
-    my $pod = $spec->generate_pod(
-    );
-    $run->out($pod);
-}
+#sub event_processed {
+#    my ($self) = @_;
+#    my $plugins = $self->spec->plugins_by_type->{GlobalOptions};
+#    for my $plugin (@$plugins) {
+#        next unless $plugin->can("event_processed");
+#        $plugin->event_processed(
+#            run => $self,
+#        );
+#    }
+#}
 
 1;
 
 __END__
 =pod
 
+=head1 NAME
+
+App::Spec::Run - App::Spec framework to run your app
+
 =head1 DESCRIPTION
 
 App::Spec::Run is the framework which runs your app defined by the spec.
-Your app class should inherit from App::Spec::Run::Cmd.
+Your app class should inherit from L<App::Spec::Run::Cmd>.
+
+=head1 SYNOPSIS
+
+    sub your_command {
+        my ($self, $run) = @_;
+        my $options = $run->options;
+        $run->out("It works");
+    }
 
 =head1 METHODS
 
 =over 4
 
+=item Constructor
+
+You can create the object yourself like this:
+
+    my $run = App::Spec::Run->new(
+        spec => $appspec,
+        cmd => App::YourApp->new,
+    );
+
+Or you use the runner method of L<App::Spec>, which will create it for you:
+
+    my $run = $appspec->runner(...);
+
+Both methods take optional arguments:
+
+    my $run = App::Spec::Run->new(
+        spec => $appspec,
+        cmd => App::YourApp->new,
+
+        # Custom array instead of the default ARGV.
+        # The contents of this array will be modified
+        argv => \@my_arguments,
+    );
+
 =item run
 
-Actually runs your app
+    $run->run;
+
+Actually runs your app. Calls C<process> and C<finish>.
+
+=item process
+
+    $run->process;
+
+Processes input, validates, runs your command and fills the
+response object.
+
+Does not print the output and does not exit.
+
+=item out
+
+    $run->out("Hello world!");
+
+Appends to response output. Adds a newline if not present
+
+=item err
+
+    $run->err("Oops, that went wrong");
+
+Appends to response error output. Adds a newline if not present
+
+=item halt
+
+    $run->halt;
+
+Further processing is halted.
+
+=item finish
+
+    $run->finish;
+
+Prints the output and exits with the exit code stored in C<response>.
+
+=item process_input
+
+    $run->process_input(
+        option_specs => \%option_specs,
+        param_specs => \%param_specs,
+    );
+
+=item process_parameters
+
+    $run->process_parameters(
+        parameter_list => $global_parameters,
+        param_specs => $param_specs,
+    );
+
+=item run_op
+
+    $run->run_op("yourcommand");
+    # shortcut for
+    $run->cmd->yourcommand($run);
+
+=item completion_output
+
+    $run->completion_output(
+        param_specs => \%param_specs,
+        completion_parameter => $completion_parameter,
+    );
+
+Is called when in completion mode
+
+=item colored
+
+Returns the given text colored, if colors are active for the given
+output.
+
+    $msg = $run->colored('err', [qw/ error /], "Oops");
+    $msg = $run->colored('out', [qw/ green /], "Everything is fine!");
+
+=item colorize
+
+    my $color_active = $run->colorize('out');
+    my $color_error_active = $run->colorize('err');
+
+Returns 1 or 0 if given output color are active. That means, the output
+is going to a terminal instead of being redirected.
+
+=item colorize_code
+
+    my $colored = $run->colorize_code('out');
+    my $text = $colored->(['green'], "Hurray");
+    # or
+    my $text = "Hurray";
+    $colored->(['green'], $text);
+
+Returns a coderef which you can use for coloring
+
+=item colorize_error
+
+    my $msg = $run->colorize_error("ouch");
+
+Returns the message in the standard error color (bold red).
+
+=item error_output
+
+    $run->error_output;
+
+Outputs any errors.
+
+Calls C<halt>
+
+=item event_globaloptions
+
+Calls any plugin that needs to know
+
+=item subscribe
+
+A plugin can subscribe for an event:
+
+    $run->subscribe(
+        print_output => {
+            plugin => $self,
+            method => "print_output",
+        },
+
+=back
+
+=head1 ATTRIBUTES
+
+=over 4
+
+=item spec
+
+Your spec, (App::Spec)
+
+=item options
+
+A hashref with the given options
+
+    {
+        verbose => 1,
+        foo => 23,
+    }
+
+=item parameters
+
+A hashref with the given parameters
+
+=item commands
+
+An arrayref containing all subcommands from the commandline
+
+=item argv_orig
+
+This contains the original contents of C<argv> before processing
+
+=item argv
+
+This is a reference to the commandline arguments array C<@ARGV>, or the
+array reference you specified otherwise. When calling your command method,
+it will contain the rest of the arguments which weren't processed as
+any subcommand, option or parameter.
+
+=item validation_errors
+
+Contains errors from option/parameter validation
+
+=item op
+
+Contains the operation (subroutine) which will be executed to run
+yor command
+
+=item cmd
+
+This is an instance of your app class
+
+=item response
+
+This contains the response of your command (exit code, output, ...)
+
+See L<App::Spec::Run::Response>
+
+=item subscribers
+
+Contains a hashref
+
+    {
+        print_output => {
+            module => $plugin,
+            mathod => 'print_output',
+        },
+    }
 
 =back
 
